/*
	Copyright 2009 Benjamin Vedder	vedder87@gmail.com
	
	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    */

package mario;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import javax.swing.*;
import javax.swing.UIManager.LookAndFeelInfo;

import mario.dataCollections.MapData;
import mario.dataCollections.TileSet;
import mario.sprites.SpriteEditor;
import mario.sprites.SpriteRenderer;
import mario.tools.AboutDialog;
import mario.tools.Tile;

public class MainWindow extends JFrame {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public static final String versionNumber = "0.006";
	
	// Self reference
	private MainWindow self = this;
	
	// Stuff
	JFileChooser fc = new JFileChooser();
	
	private MapEditor editor = new MapEditor(TestSet.TILESET, TestSet.COLLDATA, TestSet.MAP1, 60, 30, this);
	private TileEditor tEditor = new TileEditor(editor.tileset, this);
	private SpriteEditor sEditor = new SpriteEditor(this);
	
	// Components
	JTabbedPane tp = new JTabbedPane();
	
	// Menubar
	private JMenuBar menuBar = new JMenuBar();
	
	private JMenu fileMenu = new JMenu("File");
	
	private JMenuItem newMapItem = new JMenuItem("New map");
	private JMenuItem newTilesetItem = new JMenuItem("New tileset");
	private JMenuItem newSpriteItem = new JMenuItem("New sprite");
	
	private JMenuItem exportMapItem = new JMenuItem("Export map to header");
	private JMenuItem exportTileSetItem = new JMenuItem("Export tileset to header");
	private JMenuItem exportSpriteItem = new JMenuItem("Export sprites to header");
	
	private JMenuItem saveMapItem = new JMenuItem("Save map");
	private JMenuItem saveTileSetItem = new JMenuItem("Save tileset");
	private JMenuItem importMapItem = new JMenuItem("Import map from header");
	private JMenuItem importTileSetItem = new JMenuItem("Import tileset from header");
	private JMenuItem importSpriteItem = new JMenuItem("Import sprites from header");
	
	
	private JMenu helpMenu = new JMenu("Help");
	
	private JMenuItem aboutItem = new JMenuItem("About");
	
	
	private ActionListener menuListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			
			// File menu
			if (e.getSource() == newMapItem) {
				newMap();
			} 
			
			else if (e.getSource() == newTilesetItem) {
				newTileset();
			}
			
			else if (e.getSource() == newSpriteItem) {
				// TODO
			}
			
			else if (e.getSource() == exportMapItem) {
				exportMap();
			}
			
			else if (e.getSource() == exportTileSetItem) {
				exportTileset();
			}
			
			else if (e.getSource() == exportSpriteItem) {
				exportSprite();
			}

			else if (e.getSource() == saveMapItem) {
				saveTileset();
			}

			else if (e.getSource() == saveTileSetItem) {
				saveMap();
			}
			
			else if (e.getSource() == importMapItem) {
				importMap();
			}
			
			else if (e.getSource() == importTileSetItem) {
				importTileset();
			}
			
			else if (e.getSource() == importSpriteItem) {
				importSprites();
			}
			
			// Help menu
			else if (e.getSource() == aboutItem) {
				new AboutDialog(self);
			}
		}
		
	};
	
	public MainWindow() {
		setLayout(new BorderLayout());
		
		setJMenuBar(menuBar);
		menuBar.add(fileMenu);
		
		fileMenu.add(newMapItem);
		fileMenu.add(newTilesetItem);
		fileMenu.add(newSpriteItem);
		fileMenu.addSeparator();
		fileMenu.add(exportMapItem);
		fileMenu.add(exportTileSetItem);
		fileMenu.add(exportSpriteItem);
		fileMenu.addSeparator();
		fileMenu.add(saveMapItem);
		fileMenu.add(saveTileSetItem);
		fileMenu.addSeparator();
		fileMenu.add(importMapItem);
		fileMenu.add(importTileSetItem);
		fileMenu.add(importSpriteItem);
		
		menuBar.add(helpMenu);
		
		helpMenu.add(aboutItem);
		
		
		newMapItem.addActionListener(menuListener);
		newTilesetItem.addActionListener(menuListener);
		newSpriteItem.addActionListener(menuListener);
		exportMapItem.addActionListener(menuListener);
		exportSpriteItem.addActionListener(menuListener);
		exportTileSetItem.addActionListener(menuListener);
		saveMapItem.addActionListener(menuListener);
		saveTileSetItem.addActionListener(menuListener);
		importMapItem.addActionListener(menuListener);
		importTileSetItem.addActionListener(menuListener);
		importSpriteItem.addActionListener(menuListener);
		
		aboutItem.addActionListener(menuListener);
		
		
		tp.addTab("Map editor", editor);
		tp.addTab("Tile editor", tEditor);
		tp.addTab("Sprite editor", sEditor);
		
		sEditor.setSprite(TestSet.MARIO16);
		
		add(tp, BorderLayout.CENTER);
		
		setTitle("AVGA tile/map editor ALFA");
		setSize(new Dimension(1000, 650));
		setLocationRelativeTo(null);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setVisible(true);
		
	}
	
	public void addEmptyTiles(int n) {
		try {
			tp.remove(tEditor);
		} catch(Exception e) {
			
		}
		
		TileSet ts = editor.tileset;
		
		ts.addEmptyTiles(n);
		
		editor.setTileset(ts);
		
		tEditor = new TileEditor(ts, this);
		
		tp.insertTab("Tile editor", null, tEditor, "Tile editor", 1);
		tp.setSelectedIndex(1);
		
		validate();
	}
	
	public void removeTiles(int n) {
		try {
			tp.remove(tEditor);
		} catch(Exception e) {
			
		}
		
		TileSet ts = editor.tileset;
		
		ts.removeTiles(n);
		
		editor.setTileset(ts);
		
		tEditor = new TileEditor(ts, this);
		
		tp.insertTab("Tile editor", null, tEditor, "Tile editor", 1);
		tp.setSelectedIndex(1);
		
		validate();
	}

	public void newMap() {
		
		String a = JOptionPane.showInputDialog(this, 
				"Enter the dimensions, separate them with space.",
				"New map",  
				JOptionPane.OK_CANCEL_OPTION);
		
		if (a == null) {
			return;
		}
		
		int w, h;
		
		try {
			Scanner sc = new Scanner(a);
			w = sc.nextInt();
			h = sc.nextInt();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Bad input. Please check what you " +
					"have typed and try again.");
			return;
		}
		
		try {
			tp.remove(editor);
		} catch(Exception e) {
			
		}

		MapData md = new MapData(editor.tileset, w, h);
		
		editor = new MapEditor(md, this);
		
		tp.insertTab("Map editor", null, editor, null, 0);
		tp.setSelectedIndex(0);
		
		validate();
	}
	
public void newTileset() {
		
		String a = JOptionPane.showInputDialog(this, 
				"<html>Enter the number of tiles (1 - 256). <BR><font color=red>WARNING: this will destroy the current map!</font color=red></html>",
				"New tileset",  
				JOptionPane.OK_CANCEL_OPTION);
		
		if (a == null) {
			return;
		}
		
		int n;
		
		try {
			Scanner sc = new Scanner(a);
			n = sc.nextInt();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Bad input. Please check what you " +
					"have typed and try again.");
			return;
		}
		
		try {
			tp.remove(tEditor);
		} catch(Exception e) {
			
		}
		
		TileSet ts = new TileSet(n);
		
		editor.setTileset(ts);
		
		tEditor = new TileEditor(ts, this);
						
		tp.insertTab("Tile editor", null, tEditor, null, 1);
		tp.setSelectedIndex(0);
		
		validate();
	}

	public void exportMap() {
		int returnVal = fc.showSaveDialog(this);

		File file;

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			return;
		}
		
		if (file.exists()) {
			if (!(JOptionPane.showConfirmDialog(this, "Overwrite existing file?") == JOptionPane.OK_OPTION)) {
				return;
			}
		}

		try {
			String path = file.getAbsolutePath();
			if (!path.endsWith(".h")) {
				path = path + ".h";
			}

			BufferedWriter out = new BufferedWriter(new FileWriter(path));
			out.write("// Auto generated map header file by ALFA software \n\n" +

			"// Map properties: \n" +

			"// Width: " + editor.map.getMapWidth() + "\n" + "// Height: "
					+ editor.map.getMapHeight() + "\n\n" +

					"#define MAP1 { \\\n");

			int i = 1;
			for (char element : editor.map.getDataRaw()) {
				out.write(String.format("0x%X, ", (int) element));
				if (i == 12) {
					out.write("\\\n");
					i = 0;
				}
				i++;
			}

			out.write("}");

			out.close();

			JOptionPane
					.showMessageDialog(
							this,
							"<html>"
									+ "Map header file has been written to <br><font color=green>"
									+ path + "</font color=green></html>");
		} catch (IOException e) {
		}
	}

	public void exportTileset() {
		int returnVal = fc.showSaveDialog(this);

		File file;

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			return;
		}
		
		if (file.exists()) {
			if (!(JOptionPane.showConfirmDialog(this, "Overwrite existing file?") == JOptionPane.OK_OPTION)) {
				return;
			}
		}
		
		try {
			String path = file.getAbsolutePath();
			if (!path.endsWith(".h")) {
				path = path + ".h";
			}
			
	        BufferedWriter out = new BufferedWriter(new FileWriter(path));
	        out.write("// Auto generated tileset header file by ALFA software \n\n" +
	        		
	        		"// Tileset properties: \n" +
	        		
	        		"// Width: " + Tile.WIDTH + "\n" +
	        		"// Height: " + Tile.HEIGHT + "\n" +
	        		"// Tile count: " + editor.tileset.getLength() + "\n\n" +
	        		
	        		"#define TILESET { \\\n");
	        
	        int i = 1, j = 0;
	        int len = Tile.TILE_BPP * Tile.HEIGHT;
	        for (char element: editor.tileset.getRawData()) {
	        	if (j % len == 0) {
	        		out.write(" /* block : " + String.format("0x%X, ", j / len) + " */ \\\n");
	        	}
                        
                        int element2 = ((element&0xF0)>>4) | ((element&0x0F)<<4);
                                                          
	        	out.write(String.format("0x%X, ", (int)element2));
	        	if (i == Tile.TILE_BPP) {
	        		out.write("\\\n");
	        		i = 0;
	        	}
	        	i++;
	        	j++;
	        }
	        
	        out.write("}\n\n");
	        
	        // --------------- Collision data --------------------- //
	        out.write("#define COLLDATA { \\\n");
	        
	        i = 1;
			for (char element : editor.tileset.getRawCollisionData()) {
				out.write(String.format("0x%X, ", (int) element));
				if (i == 8) {
					out.write("\\\n");
					i = 0;
				}
				i++;
			}
			out.write("}\n");
	        // ---------------------------------------------------- //
	        
	        
	        
	        out.close();
	        
	        JOptionPane.showMessageDialog(this, "<html>" +
	        		"Tileset header file has been written to <br><font color=green>" +
	        		path +
	        		"</font color=green></html>");
	    } catch (IOException e) {
	    }
	}
	
	public void exportSprite() {
		int returnVal = fc.showSaveDialog(this);

		File file;

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			return;
		}
		
		if (file.exists()) {
			if (!(JOptionPane.showConfirmDialog(this, "Overwrite existing file?") == JOptionPane.OK_OPTION)) {
				return;
			}
		}

		try {
			String path = file.getAbsolutePath();
			if (!path.endsWith(".h")) {
				path = path + ".h";
			}

			BufferedWriter out = new BufferedWriter(new FileWriter(path));
			out.write("// Auto generated map header file by ALFA software \n\n" +

			"// Sprite properties: \n" +

			"// Width: " + SpriteRenderer.SPRITES_X + "\n" + "// Height: "
					+ SpriteRenderer.SPRITES_Y + "\n\n" +

					"#define MARIO16 { \\\n");

			int i = 1;
			for (char element : sEditor.getRawData()) {
				out.write(String.format("0x%X, ", (int) element));
				if (i == 8) {
					out.write("\\\n");
					i = 0;
				}
				i++;
			}

			out.write("}");

			out.close();

			JOptionPane
					.showMessageDialog(
							this,
							"<html>"
									+ "Sprite header file has been written to <br><font color=green>"
									+ path + "</font color=green></html>");
		} catch (IOException e) {
		}
	}

	public void saveMap() {
		JOptionPane.showMessageDialog(null, "Not implemented yet... Use export map instead, it can be imported in that way too.");
	}

	public void saveTileset() {
		JOptionPane.showMessageDialog(null, "Not implemented yet... Use export tileset instead, it can be imported in that way too.");
	}

	public void importMap() {
		int returnVal = fc.showOpenDialog(this);

		File file;

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			return;
		}
		
		if (!file.exists()) {
			if (!(JOptionPane.showConfirmDialog(this, "File not found...") == JOptionPane.OK_OPTION)) {
				return;
			}
		}
		
		int w = 0;
		int h = 0;
		List<Character> data = new ArrayList<Character>();
		
		try {
			FileInputStream fstream = new FileInputStream(file);
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			
			String line;
			while ((line = br.readLine()) != null) {
				Scanner sc = new Scanner(line);
				
				while (sc.hasNext() && w == 0) {
					if (sc.next().indexOf("Width:") >= 0) {
						try {
							w = sc.nextInt();
						} catch (Exception exc) {
							JOptionPane.showMessageDialog(this, "Error parsing width from file!!!");
						}
						break;
					}
				}
				
				while (sc.hasNext() && h == 0) {
					if (sc.next().indexOf("Height:") >= 0) {
						try {
							h = sc.nextInt();
						} catch (Exception exc) {
							JOptionPane.showMessageDialog(this, "Error parsing height from file!!!");
						}
						break;
					}
				}
								
				if (w != 0 && h != 0) {
					while ((line = br.readLine()) != null) {
						if (line.indexOf("#define") >= 0) {
							while ((line = br.readLine()) != null) {
								sc = new Scanner(line);
								
								while (sc.hasNext()) {
									String next;
									if ((next = sc.next()).toLowerCase().startsWith("0x")) {
										int lastIndex = next.length();
										int tmp = next.lastIndexOf(",");
										if (tmp > -1) {
											lastIndex = tmp;
										}
										try {
											data.add((char)(Integer.parseInt(next.substring(2, lastIndex), 16)));
										} catch (Exception e) {
											JOptionPane.showMessageDialog(this, "Error parsing hex from text file");
										}
									}
								}
							}
							
							break;
						}
					}
					break;
				}
			}
			in.close();
			
		} catch (Exception e) {// Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}
		
		try {
			tp.remove(editor);
		} catch(Exception e) {
			
		}
		
		char[] dataArray = new char[data.size()];
		
		for (int i = 0;i < data.size();i++) {
			dataArray[i] = data.get(i);
		}
		
		MapData md = new MapData(editor.tileset, dataArray, w, h);
		
		editor = new MapEditor(md, this);
		
		tp.insertTab("Map editor", null, editor, null, 0);
		tp.setSelectedIndex(0);
		
		validate();
	}

	public void importTileset() {
		int returnVal = fc.showOpenDialog(this);

		File file;

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			return;
		}
		
		if (!file.exists()) {
			if (!(JOptionPane.showConfirmDialog(this, "File not found...") == JOptionPane.OK_OPTION)) {
				return;
			}
		}
		
		int n = 0;
		boolean dataFound = false;
		List<Character> data = new ArrayList<Character>();
		List<Character> colData = new ArrayList<Character>();
		
		try {
			FileInputStream fstream = new FileInputStream(file);
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			
			String line;
			while ((line = br.readLine()) != null) {
				Scanner sc = new Scanner(line);
				
				while (sc.hasNext() && n == 0) {
					if (sc.next().indexOf("count:") >= 0) {
						try {
							n = sc.nextInt();
						} catch (Exception exc) {
							JOptionPane.showMessageDialog(this, "Error parsing tile count from file!!!");
						}
						break;
					}
				}
				
								
				if (n > 0) {
					while ((line = br.readLine()) != null) {
						if (line.indexOf("#define") >= 0 && !dataFound) {
							while ((line = br.readLine()) != null) {
								if (line.indexOf("/*") >= 0) {
									continue;
								}
								if (line.indexOf("}") >= 0) {
									dataFound = true;
									break;
								}
								sc = new Scanner(line);
								
								while (sc.hasNext()) {
									String next;
									if ((next = sc.next()).toLowerCase().startsWith("0x")) {
										int lastIndex = next.length();
										int tmp = next.lastIndexOf(",");
										if (tmp > -1) {
											lastIndex = tmp;
										}
										try {
											int a = (Integer.parseInt(next.substring(2, lastIndex), 16));
                                                                                        int b = ((a&0xF0)>>4) | ((a&0x0F)<<4);
                                                                                        data.add((char)b);
										} catch (Exception e) {
											JOptionPane.showMessageDialog(this, "Error parsing hex from text file");
										}
									}
								}
							}
						}
						// --------- Collision data ------------ //
						if (line.indexOf("#define") >= 0 && dataFound) {
							while ((line = br.readLine()) != null) {
								if (line.indexOf("/*") >= 0) {
									continue;
								}
								if (line.indexOf("}") >= 0) {
									break;
								}
								sc = new Scanner(line);
								
								while (sc.hasNext()) {
									String next;
									if ((next = sc.next()).toLowerCase().startsWith("0x")) {
										int lastIndex = next.length();
										int tmp = next.lastIndexOf(",");
										if (tmp > -1) {
											lastIndex = tmp;
										}
										try {
											colData.add((char)(Integer.parseInt(next.substring(2, lastIndex), 16)));
										} catch (Exception e) {
											JOptionPane.showMessageDialog(this, "Error parsing hex from text file");
										}
									}
								}
							}
							
							break;
						}
						// ------------------------------------- //
					}
					break;
				}
			}
			in.close();
			
		} catch (Exception e) {// Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}
		
		
		char[] dataArray = new char[data.size()];
		
		for (int i = 0;i < data.size();i++) {
			dataArray[i] = data.get(i);
		}
		
		try {
			tp.remove(tEditor);
		} catch(Exception e) {
			
		}
		
		
		TileSet ts;
		
		if (colData.size() > 0) {
			char[] colDataArray = new char[data.size()];
			
			for (int i = 0;i < colData.size();i++) {
				colDataArray[i] = colData.get(i);
			}
			
			ts = new TileSet(dataArray, colDataArray);
		} else {
			ts = new TileSet(dataArray, TestSet.COLLDATA);
		}
		
		editor.setTileset(ts);
		
		tEditor = new TileEditor(ts, this);
		
		tp.insertTab("Tile editor", null, tEditor, "Tile editor", 1);
		tp.setSelectedIndex(0);
		
		validate();
	}
	
	public void importSprites() {
		int returnVal = fc.showOpenDialog(this);

		File file;

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
		} else {
			return;
		}
		
		if (!file.exists()) {
			if (!(JOptionPane.showConfirmDialog(this, "File not found...") == JOptionPane.OK_OPTION)) {
				return;
			}
		}
		
//		int n = 0;
		List<Character> data = new ArrayList<Character>();
		
		try {
			FileInputStream fstream = new FileInputStream(file);
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			
			String line;
			while ((line = br.readLine()) != null) {
				Scanner sc = new Scanner(line);
				
//				while (sc.hasNext() && n == 0) {
//					if (sc.next().indexOf("count:") >= 0) {
//						try {
//							n = sc.nextInt();
//						} catch (Exception exc) {
//							JOptionPane.showMessageDialog(this, "Error parsing tile count from file!!!");
//						}
//						break;
//					}
//				}
				
								
				if (true) {
					while ((line = br.readLine()) != null) {
						if (line.indexOf("#define") >= 0) {
							while ((line = br.readLine()) != null) {
								if (line.indexOf("/*") >= 0) {
									continue;
								}
								sc = new Scanner(line);
								
								while (sc.hasNext()) {
									String next;
									if ((next = sc.next()).toLowerCase().startsWith("0x")) {
										int lastIndex = next.length();
										int tmp = next.lastIndexOf(",");
										if (tmp > -1) {
											lastIndex = tmp;
										}
										try {
											data.add((char)(Integer.parseInt(next.substring(2, lastIndex), 16)));
										} catch (Exception e) {
											JOptionPane.showMessageDialog(this, "Error parsing hex from text file");
										}
									}
								}
							}
							
							break;
						}
					}
					break;
				}
			}
			in.close();
			
		} catch (Exception e) {// Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}
		
		char[] dataArray = new char[data.size()];
		
		for (int i = 0;i < data.size();i++) {
			dataArray[i] = data.get(i);
		}
		
		sEditor.setSprite(dataArray);
		
		tp.setSelectedIndex(tp.getTabCount() - 1);
		
		validate();
	}
	
	public static void main(String[] arg) {
		try {
			for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
//			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (UnsupportedLookAndFeelException e) {
			// handle exception
		} catch (ClassNotFoundException e) {
			// handle exception
		} catch (InstantiationException e) {
			// handle exception
		} catch (IllegalAccessException e) {
			// handle exception
		}

		new MainWindow();
	}
}