/*
	Copyright 2009 Benjamin Vedder	vedder87@gmail.com
	
	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    */

package mario.sprites;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import mario.tools.ColorButton;
import mario.tools.Tools;
import mario.tools.tileEditor.ColorReplaceDialog;

import javax.swing.*;
import javax.swing.border.EtchedBorder;

public class SpriteEditor extends JPanel implements ActionListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public static final int SPRITES_PER_SET = 8;
	
	private JFrame parent = null;
		
	private ColorReplaceDialog replaceDialog;
	
	private List<SpriteRenderer> renderers = new ArrayList<SpriteRenderer>();
	
	// Components
	private JPanel tool1Panel = new JPanel(new BorderLayout());
	private JPanel tool2Panel = new JPanel(new BorderLayout());
	
	private JPanel colorBar = new JPanel(new GridLayout(2, 8));
	private JPanel currentColor = new JPanel();
	
	private JPanel spritePanel = null;

	private JButton replaceColorButton = new JButton("Replace all colors...");
	private JButton saveAllButton = new JButton("Save all");
	
	public SpriteEditor(JFrame parent) {
		this.parent = parent;
		
		init();
	}
	
	private void init() {
		spritePanel = new JPanel(new GridLayout(2, SPRITES_PER_SET / 2));
		
		setLayout(new BorderLayout());
		
		replaceColorButton.addActionListener(this);
		saveAllButton.addActionListener(this);
		
		for (int i = 0;i < 16;i++) {
			ColorButton b = new ColorButton();
			b.setOpaque(true);
			b.setBackground(Tools.getColor(i));
			b.setPreferredSize(new Dimension(32, 32));
			
			b.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					ColorButton b = (ColorButton)arg0.getSource();
					currentColor.setBackground(b.getBackground());
				}
			});
			
			colorBar.add(b);
		}
		
		tool1Panel.add(colorBar, BorderLayout.CENTER);
		
		JPanel tmpPanel = new JPanel(new BorderLayout());
		tmpPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		
		currentColor.setOpaque(true);
		currentColor.setBackground(Tools.getColor(0));
		currentColor.setPreferredSize(new Dimension(80, 80));
		tmpPanel.add(currentColor, BorderLayout.CENTER);
		
		tool1Panel.add(tmpPanel, BorderLayout.EAST);
		
		tool2Panel.add(replaceColorButton, BorderLayout.NORTH);
		
		tool1Panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Palette"));
		tool2Panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Tools"));
		
		for (int i = 0;i < SPRITES_PER_SET;i++) {
			SpriteRenderer rend = new SpriteRenderer(this);
			rend.setBorder(BorderFactory.createEtchedBorder());
			
			spritePanel.add(rend);
			renderers.add(rend);
		}
		
		JPanel tmp = new JPanel(new BorderLayout());
		tmp.add(spritePanel, BorderLayout.SOUTH);
		
		add(tmp, BorderLayout.WEST);
		add(tool1Panel, BorderLayout.SOUTH);
		add(tool2Panel, BorderLayout.EAST);
	}
	
	public void replaceColor(Color oldColor, Color newColor) {
		// TODO!
		
		if (parent != null) {
			parent.validate();
		}
		repaint();
	}
	
	public Color getCurrentColor() {
		return currentColor.getBackground();
	}
	
	public void setCurrentColor(Color c) {
		currentColor.setBackground(c);
	}
	
	public void setSprite(char[] sp) {
		int ind = 0, ind2 = 0;
		
		int pic = (SpriteRenderer.SPRITES_X * SpriteRenderer.SPRITES_Y) / 2;
		char[] tmp = new char[pic];
		
		for (char element: sp) {
			if (ind == pic) {
				ind = 0;
				
				if (ind2 < renderers.size()) {
					renderers.get(ind2).setSprite(tmp);
				}
				
				ind2++;
				tmp = new char[pic];
			}
			
			tmp[ind] = element;
			
			ind++;
		}
		
		// Add the last sprite.
		if (ind2 < renderers.size()) {
			renderers.get(ind2).setSprite(tmp);
		}
	}
	
	public char[] getRawData() {
		List<Character> data = new ArrayList<Character>();
		
		for (SpriteRenderer element: renderers) {
			char[] tmp = element.getSprite();
			
			boolean add = false;
			char a = tmp[0];
			for (char element2: tmp) {
				if (element2 != a) {
					add = true;
				}
			}
			
			if (add) {
				for (char element2: tmp) {
					data.add(element2);
				}
			}
		}
		
		char[] array = new char[data.size()];
		for (int i = 0;i < data.size();i++) {
			array[i] = data.get(i);
		}
		
		return array;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == replaceColorButton) {
			replaceDialog = new ColorReplaceDialog(parent);
			replaceDialog.addActionListener(this);
			
		} else if (e.getSource() == replaceDialog) {
			replaceColor(replaceDialog.getOldColor(), replaceDialog.getNewColor());
		}
	}
}








